Renderer REND;
PhysWorld WORLD;
Stabilizer FRS;
Pointer DRAGGED;
MouseController MC;

// Polygon shapes
Point[] box;

// Various materials
Material alu;
Material wood;
Material fixed;
Material iron;
Material plastic;

// Physical objects
Pointer b0;
Pointer b1;
Pointer b2;
Pointer b4;
Pointer b5;
Pointer b6;
Pointer b7;
Pointer b8;

// Weld Points
Pointer w0b1;
Pointer w0b2;
Pointer w0b5;
Pointer w1b5;

void setup()
{  
  size(800,420);
  smooth();

  box = new Point[4];
  box[0] = new Point(+1,-1);
  box[1] = new Point(+1,+1);
  box[2] = new Point(-1,+1);
  box[3] = new Point(-1,-1);
  
  FRS = new Stabilizer(2);
  REND = new Renderer();
  WORLD = new PhysWorld();
 
  //==================================
  // CHANGE THESE
  //==================================

  // Gravity
  WORLD.setGravity(9.81,HALF_PI);

  // Various materials
  alu = new Material(300, 0.3, 10, #F1F1F1);
  wood = new Material(400, 0.1, 15, #CC9966);
  fixed = new Material(0, 0.9, 10, #000000);
  iron = new Material(1000, 0.3, 9, #999999);
  plastic = new Material(300, 0.2, 8, #94D6FC);

  // Physical objects
  b0 = WORLD.addObject(pix2met(400),pix2met(431),fixed,new Polygon(box).scale(pix2met(800),pix2met(25)));
  b1 = WORLD.addObject(pix2met(810),pix2met(0),fixed,new Polygon(box).scale(pix2met(18),pix2met(840)));
  b2 = WORLD.addObject(pix2met(-10),pix2met(0),fixed,new Polygon(box).scale(pix2met(17),pix2met(840)));
  b4 = WORLD.addObject(pix2met(500),pix2met(100),plastic,new Polygon(3).scale(pix2met(180),pix2met(120)));
  b5 = WORLD.addObject(pix2met(400),pix2met(200),iron,new Polygon(box).scale(pix2met(300),pix2met(30)));
  b6 = WORLD.addObject(pix2met(500),pix2met(100),alu,new Polygon(4).scale(pix2met(150),pix2met(100)));
  b7 = WORLD.addObject(pix2met(300),pix2met(100),wood,new Polygon(4).scale(pix2met(150),pix2met(148)));
  b8 = WORLD.addObject(pix2met(400),pix2met(-10),fixed,new Polygon(box).scale(pix2met(800),pix2met(17)));

  // Weld Points
  w0b1 = WORLD.addWeldPoint(b1,new Point(pix2met(-50),pix2met(+50)));
  w0b2 = WORLD.addWeldPoint(b2,new Point(pix2met(+50),pix2met(+50)));
  w0b5 = WORLD.addWeldPoint(b5,new Point(pix2met(+130),pix2met(0)));
  w1b5 = WORLD.addWeldPoint(b5,new Point(pix2met(-130),pix2met(0)));

  // Links
  WORLD.addLink(b1,w0b1,b5,w0b5,pix2met(280),35000);
  WORLD.addLink(b2,w0b2,b5,w1b5,pix2met(280),35000);

  //==================================
  // STOP CHANGING! ;)
  //==================================
  
  MC = new MouseController();
}

void draw()
{
  REND.drawFrame();
  WORLD.update();
  FRS.update(framerate);
}

float pix2met(float Pixels)
{
  return Pixels/51;
}

float met2pix(float Meters)
{
  return Meters*51;
}

void mousePressed()
{
  MC.pressed();
}

void mouseDragged()
{
  MC.dragged();
}

void mouseReleased()
{
  MC.released();
}
