class PhysObject
{
  Polygon poly;
  Material material;
  
  Force[] forces;
  Vector accel;
  Vector veloc;
  Point center;
  
  float torque;
  float invMomentOfInertia;
  float rotationSpeed;
  
  Point[] weldPoints;

  PhysObject()
  {
    forces = new Force[5];
    for (int i=0; i<forces.length; i++) forces[i] = new Force();
    
    weldPoints = new Point[10];
    for (int i=0; i<weldPoints.length; i++) weldPoints[i] = new Point(-1.0,-1.0);
    
    accel = new Vector();
    veloc = new Vector();
    rotationSpeed = 0;
    invMomentOfInertia = 1;
  }

  void update(Vector gravity, float delta_t)
  {
    fall(gravity);
    move(delta_t);
  }

  void reset()
  {
    for (int i=0; i<forces.length; i++)
    {
      forces[i] = new Force();
    }
    
    accel = new Vector();
    torque = 0;
  }

  void fall(Vector gravity)
  {
    if ( material.invDensity != 0 )
    {
      float mass = poly.area/material.invDensity;
      addForce(new Point(0,0), gravity.multiply(mass));
    }
  }

  void move(float delta_t)
  {
    Vector totalForceVector = new Vector();   
    for (int i=0; i<forces.length; i++)
    {
      totalForceVector = totalForceVector.addVector(forces[i].force);
    }

    float mass = poly.area/material.invDensity;
    accel = totalForceVector.multiply(1/mass);
    center = center.addVector(veloc.multiply(delta_t)).addVector(accel.multiply(pow(delta_t,2)/2));
    veloc = veloc.addVector(accel.multiply(delta_t));    
    
    for (int i=0; i<forces.length; i++)
    {
      Vector r = new Vector(forces[i].point);
      float perpDot = forces[i].force.normal().dot(r);
      torque += perpDot;
    }
    
    torque *= invMomentOfInertia;

    poly.rotatePoly(rotationSpeed*delta_t+torque*(pow(delta_t,2)/2));
    rotateWeldPoints(rotationSpeed*delta_t+torque*(pow(delta_t,2)/2));
    rotationSpeed += torque*delta_t;
    
    // Hack to reduce spinning - does the job of air friction
    rotationSpeed *= 0.99;
    // End hack
  }
  
  void rotateWeldPoints(float r)
  {
    for (int i=0; i<weldPoints.length; i++)
    {
      if ( weldPoints[i].x != -1 || weldPoints[i].y != -1 )
      {
        weldPoints[i] = weldPoints[i].rotatePoint(r);
      }
    }
  }

  void impulse(Vector v, float k, float j, Vector r)
  { 
    float invMass = material.invDensity/poly.area;
    
    center = center.addVector(v.multiply(k));
    veloc = veloc.addVector(v.multiply(j*invMass));
    rotationSpeed -= v.multiply(j).dot(r.normal())*invMomentOfInertia;
  }
  
  void addForce(Point P, Vector V)
  {
    for (int i=0; i<forces.length; i++)
    {
      if ( forces[i].force.len == 0 )
      {
        forces[i] = new Force();
        forces[i].point = P;
        forces[i].force = V;
        break;
      }
    }
  }
  
  void calculateMoment()
  {
    invMomentOfInertia = 0;
    
    if ( material.invDensity != 0 )
    {
      float R = 0;
      
      for (int i=0; i<poly.vertices.length; i++)
      {
        Vector Pn;
        Vector Pnp1;
        
        if (i<poly.vertices.length-1)
        {
          Pn = new Vector(poly.vertices[i]);
          Pnp1 = new Vector(poly.vertices[i+1]);
        }
        else
        {
          Pn = new Vector(poly.vertices[i]);
          Pnp1 = new Vector(poly.vertices[0]);
        }
        
        R += (abs(Pn.dot(Pnp1.normal()))*(Pn.dot(Pn)+Pnp1.dot(Pnp1)+Pn.dot(Pnp1)));
      }
      
      invMomentOfInertia = 1/(R/material.invDensity/12);
    }
  }
  
  boolean inObj(Point p)
  {
    float x_rel = p.x - center.x;
    float y_rel = p.y - center.y;
    
    return poly.inPoly(new Point(x_rel, y_rel));
  }
}
