class Polygon
{
  float radius;
  float area;
  Point[] vertices;

  Polygon(Point[] Vertices)
  {
    vertices = new Point[Vertices.length];
    for (int i=0; i<Vertices.length; i++) setVertex(i, Vertices[i]);
    radius = calculateRadius();
    area = calculateArea();
    translateToCenter();
  }
  
  Polygon(int num_segments)
  {
    vertices = new Point[num_segments];
    for(int i=0; i<num_segments; i++)
    {
      setVertex(i, new Point(cos(i*TWO_PI/num_segments),sin(i*TWO_PI/num_segments)));
    }
    radius = calculateRadius();
    area = calculateArea();
  }
  
  void setVertex(int i, Point vertex)
  {
    vertices[i] = vertex;
  }
  
  int numEdges()
  {
    return vertices.length;
  }
  
  Polygon scale(float w, float h)
  {
    Polygon R = new Polygon(vertices);
    for (int i=0; i<vertices.length; i++)
    {
      R.setVertex(i, vertices[i].multiply(w/2, h/2));
    }
    R.radius = R.calculateRadius();
    R.area = R.calculateArea();
    return R;
  }
  
  void rotatePoly(float r)
  {    
    for (int i=0; i<vertices.length; i++)
    {
      setVertex(i, vertices[i].rotatePoint(r));
    }
  }
  
  Vector getEdge(int index)
  {
    int nextIndex = index+1;
    
    if ( index == numEdges()-1 )
    {
      nextIndex = 0;
    }
    
    return new Vector(vertices[nextIndex].x-vertices[index].x,vertices[nextIndex].y-vertices[index].y);
  }
  
  Point getEdgeStart(int index)
  {
    return vertices[index];
  }
  
  float calculateArea()
  {
    float R = 0;
    for (int i=0; i<vertices.length; i++)
    {
      if (i < vertices.length-1)
      {
        R += (vertices[i].x*vertices[i+1].y-vertices[i+1].x*vertices[i].y)/2;
      }
      else
      {
        R += (vertices[i].x*vertices[0].y-vertices[0].x*vertices[i].y)/2;
      }
    }
    
    return abs(R);
  }
  
  void translateToCenter()
  {
    Point centerOfMass = new Point();
    
    for (int i=0; i<vertices.length; i++)
    {
      if (i < vertices.length-1)
      {
        float common = (vertices[i].x*vertices[i+1].y-vertices[i+1].x*vertices[i].y)/6/area;
        centerOfMass.x += (vertices[i].x+vertices[i+1].x)*common;
        centerOfMass.y += (vertices[i].y+vertices[i+1].y)*common;
      }
      else
      {
        float common = (vertices[i].x*vertices[0].y-vertices[0].x*vertices[i].y)/6/area;
        centerOfMass.x += (vertices[i].x+vertices[0].x)*common;
        centerOfMass.y += (vertices[i].y+vertices[0].y)*common;
      }
    }
    
    for (int i=0; i<vertices.length; i++)
    {
      vertices[i] = vertices[i].addVector(new Vector(centerOfMass).multiply(-1));
    }
  }
  
  float calculateRadius()
  {
    float R = 0;
    
    for (int i=0; i<vertices.length; i++)
    {
      float l = sqrt(pow(vertices[i].x,2)+pow(vertices[i].y,2));
      
      if (l >= R)
      {
        R = l;
      }
    }
    
    return R;
  }
  
  // Thanks to Seltar (seltar.wliia.org) and Philippe Reverdy
  boolean inPoly(Point p)
  {
    float R=0;
    
    for (int i=0; i<vertices.length; i++) 
    {
      float p1x = vertices[i].x - p.x;
      float p1y = vertices[i].y - p.y;
      float p2x = vertices[(i+1)%vertices.length].x - p.x;
      float p2y = vertices[(i+1)%vertices.length].y - p.y;
      
      R += angle2D(new Point(p1x,p1y),new Point(p2x,p2y));
    }

    if (abs(R) < PI)
      return false;
    else
      return true;
  }

  // Thanks to Seltar (seltar.wliia.org) and Philippe Reverdy
  float angle2D(Point p1, Point p2)
  {
    float theta1 = atan2(p1.y,p1.x);
    float theta2 = atan2(p2.y,p2.x);
    float dtheta = theta2 - theta1;
    
    while (dtheta > PI)
      dtheta -= TWO_PI;
    while (dtheta < -PI)
      dtheta += TWO_PI;

    return dtheta;
  }
}
